use game::*;
use bitboard::*;
use movegen::*;

fn value_castling_rights(game: &Game, side: Side) -> i32 {
    let mut val: i32 = 0;

    for i in 0..2 {
        if game.castling_rights[i] { val += 12; }
    }
    for i in 2..4 {
        if game.castling_rights[i] { val -= 12; }
    }
    
    if side == BLACK {
        return -val;
    }
    else {
        return val;
    }
}

fn side_value(game: &Game, side: Side) -> u32 {
    let knights = game.get_piece(KNIGHT, side);
    let ks = BitboardIterator { board: knights };
    let mut k_attacks: u32 = 0;

    for k in ks {
        let targets = get_knight_targets(square(k));
        k_attacks += targets.count_ones() * 4;
    }

    let adv_pawn_mask = match side { WHITE => ROW_7, BLACK => ROW_2 };
    let advanced_pawns = (game.get_piece(PAWN, side) & adv_pawn_mask).count_ones() * 200;

    advanced_pawns + k_attacks
    + game.get_piece(PAWN, side).count_ones() * 100
    + knights.count_ones() * 300
    + game.get_piece(BISHOP, side).count_ones() * 320
    + game.get_piece(ROOK, side).count_ones() * 400
    + game.get_piece(QUEEN, side).count_ones() * 700
    + game.get_piece(KING, side).count_ones() * 100000
}

pub fn evaluate(game: &Game) -> i32 {
    return side_value(game, game.turn) as i32 - side_value(game, !game.turn) as i32 + value_castling_rights(game, game.turn);
}
